%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generates Allais reversals
%
% Inputs are two pairs of lotteries L1, and L2, x and rho.
% Used for generating tables 6 and 7
%
% gamma_star is the value of gamma such that consumer is indifferent when
% GAMMA = 0. Calculated from the first pair of lotteries, but irrelevant if
% it is an Allais pair. See pp 23.
%
% mu_star is just the mu from gamma_star
% gamma_reversal and GAMMA_reversal are actually the parameters that
% generate the reversal.
%
% mu_1 and mu_2 are the pairs of mus from each situation. They are used to
% make the tables later.


function [gamma_star mu_star gamma_reversal GAMMA_reversal mu_1 mu_2] = reversal(x,rho,L1,L2)

% Start the parameters
    param.rho   = rho;
    param.x     = x;

    param.delta = 1 - 0.01/365;
    % param.delta = 1.01 .^ (-1/365);

% Calculate gamma_star
    param.GAMMA = 0;

    param.gamma = 1;
    gl = 0;
    while(decision(param,L1)==1)
        gl = param.gamma;
        param.gamma=2*param.gamma;
        gh = param.gamma;
    end;

    while(gh-gl>10^(-1))
        param.gamma = (gh+gl)/2;
        if decision(param,L1)==1
            gl = param.gamma;
        else
            gh = param.gamma;
        end;
    end;

    gamma_star = param.gamma;

    mu_star = (1+gamma_star)^(1/rho);

% Find the reversal

    param.gamma = 0.95* gamma_star;   % Take a gamma 5% below gamma_star, and then look for the reversal.
    Gl=0;
    param.GAMMA = 1;

    while(decision(param,L1)==1)
        param.GAMMA=2*param.GAMMA;
        Gh = param.GAMMA;
    end;

    while ~(decision(param,L1)==2 && decision(param,L2)==1)       %For this to work, decisions must be ordered as 11 21 22 as GAMMA grows. I don't know if its true in general, but always worked with me so far. Note that if it fails the program loops forever, but does not give a wrong solution.
        param.GAMMA = (Gh+Gl)/2;
        if decision(param,L1)==1
            Gl = param.GAMMA;
        end;
        if decision(param,L2)==2
            Gh = param.GAMMA;
        end;
    end;

    gamma_reversal = param.gamma;
    GAMMA_reversal = param.GAMMA;

% And the output
    [choice1 C1 gamma1_1 mu_1] = decision(param,L1);
    [choice2 C2 gamma1_2 mu_2] = decision(param,L2);